@@ -1,5 +1,5 @@
 import os
-from typing import List
+from typing import List, Dict
 
 from otlmow_modelbuilder.DatatypeBuilderFunctions import get_single_field_from_type_uri, get_type_link_from_attribuut, \
     get_type_name_of_complex_attribuut, get_type_name_of_union_attribuut, get_field_name_from_type_uri, \
@@ -34,7 +34,7 @@ def cardinality_check(attribute, type_string, datablock):
         return type_string
 
 
-def get_type_hint_from_field(attribute, datablock):
+def get_type_hint_from_field(attribute, datablock, valid_uri_and_types):
     field_name = get_single_field_from_type_uri(attribute.type)
     if field_name == 'StringField' or field_name == 'URIField':
         return f' -> {cardinality_check(attribute, "str", datablock)}'
@@ -54,7 +54,7 @@ def get_type_hint_from_field(attribute, datablock):
         import_datetime(datablock, 'time')
         return f' -> {cardinality_check(attribute, "time", datablock)}'
     else:
-        field_name = get_non_single_field_from_type_uri(attribute.type)
+        field_name = get_non_single_field_from_type_uri(attribute.type, valid_uri_and_types)
         if field_name[0] == 'KeuzelijstField':
             return f' -> {cardinality_check(attribute, "str", datablock)}'
         if field_name[0] == 'ComplexField' or field_name[0] == 'UnionTypeField':
@@ -73,7 +73,8 @@ def get_type_hint_from_field(attribute, datablock):
     return ''
 
 
-def add_attributen_to_data_block(attributen, datablock: List[str], for_class_use=False, type_field=''):
+def add_attributen_to_data_block(attributen, datablock: List[str], valid_uri_and_types: Dict, for_class_use=False,
+                                 type_field=''):
     prop_datablock = []
     for attribuut in sorted(attributen, key=lambda a: a.name):
         if attribuut.overerving == 1:
@@ -82,6 +83,10 @@ def add_attributen_to_data_block(attributen, datablock: List[str], for_class_use
         whitespace = get_white_space_equivalent(f'        self._{attribuut.name} = OTLAttribuut(')
         field_name = get_single_field_from_type_uri(attribuut.type)
 
+        if attribuut.objectUri == 'https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Segmentcontroller.beveil?igingssleutel':
+            attribuut.objectUri = attribuut.objectUri.replace('?', '_')
+            attribuut.name = attribuut.name.replace('?', '_')
+
         datablock.append(f'        self._{attribuut.name} = OTLAttribuut(field={field_name},')
         datablock.append(f'{whitespace}naam={wrap_in_quotes(attribuut.name)},')
         datablock.append(f'{whitespace}label={wrap_in_quotes(attribuut.label)},')
@@ -107,7 +112,7 @@ def add_attributen_to_data_block(attributen, datablock: List[str], for_class_use
         if not for_class_use:
             owner_self += '._parent'
 
-        type_hint = get_type_hint_from_field(attribuut, datablock)
+        type_hint = get_type_hint_from_field(attribuut, datablock, valid_uri_and_types)
 
         prop_datablock.append(f'    @property'),
         prop_datablock.append(f'    def {attribuut.name}(self){type_hint}:'),
@@ -132,7 +137,8 @@ def add_attributen_to_data_block(attributen, datablock: List[str], for_class_use
     return datablock
 
 
-def get_fields_to_import_from_list_of_attributes(oslo_collector, attributen, list_to_start_from=None):
+def get_fields_to_import_from_list_of_attributes(oslo_collector, attributen, valid_uri_and_types,
+                                                 list_to_start_from=None):
     if list_to_start_from is None:
         list_to_start_from = []
     if len(attributen) == 0:
@@ -148,7 +154,8 @@ def get_fields_to_import_from_list_of_attributes(oslo_collector, attributen, lis
         elif type_link == "OSLODatatypePrimitive":
             collected_list.append(get_single_field_from_type_uri(attribuut.type))
         elif type_link == "OSLODatatypeComplex":
-            collected_list.append(get_type_name_of_complex_attribuut(attribuut.type))
+            collected_list.append(get_type_name_of_complex_attribuut(attribuut.type,
+                                                                     valid_uri_and_types=valid_uri_and_types))
         elif type_link == "OSLODatatypeUnion":
             collected_list.append(get_type_name_of_union_attribuut(attribuut.type))
         else:
